@@ -12,9 +12,6 @@ from __future__ import absolute_import
 import logging
 import re
 
-# Import Salt libs
-import salt.utils
-
 # Import Third Party Libs
 try:
     import psutil
@@ -28,9 +25,7 @@ __virtualname__ = 'diskusage'
 
 
 def __virtual__():
-    if salt.utils.is_windows():
-        return False
-    elif HAS_PSUTIL is False:
+    if HAS_PSUTIL is False:
         return False
     else:
         return __virtualname__
@@ -60,10 +55,20 @@ def beacon(config):
           diskusage:
             - /: 63%
             - /mnt/nfs: 50%
+
+    Windows drives must be quoted to avoid yaml syntax errors
+
+    .. code-block:: yaml
+
+        beacons:
+          diskusage:
+            -  interval: 120
+            - 'c:\': 90%
+            - 'd:\': 50%
+
     '''
     ret = []
-    for diskusage in config:
-        mount = diskusage.keys()[0]
+    for mount in config:
 
         try:
             _current_usage = psutil.disk_usage(mount)
@@ -73,7 +78,7 @@ def beacon(config):
             continue
 
         current_usage = _current_usage.percent
-        monitor_usage = diskusage[mount]
+        monitor_usage = config[mount]
         if '%' in monitor_usage:
             monitor_usage = re.sub('%', '', monitor_usage)
         monitor_usage = float(monitor_usage)
