@@ -246,7 +246,8 @@ def create(vm_):
         # Check for required profile parameters before sending any API calls.
         if vm_['profile'] and config.is_profile_configured(__opts__,
                                                            __active_provider_name__ or 'softlayer',
-                                                           vm_['profile']) is False:
+                                                           vm_['profile'],
+                                                           vm_=vm_) is False:
             return False
     except AttributeError:
         pass
@@ -256,23 +257,37 @@ def create(vm_):
     if 'provider' in vm_:
         vm_['driver'] = vm_.pop('provider')
 
+    name = vm_['name']
+    hostname = name
+    domain = config.get_cloud_config_value(
+        'domain', vm_, __opts__, default=None
+    )
+    if domain is None:
+        SaltCloudSystemExit(
+            'A domain name is required for the SoftLayer driver.'
+        )
+
+    if vm_.get('use_fqdn'):
+        name = '.'.join([name, domain])
+        vm_['name'] = name
+
     salt.utils.cloud.fire_event(
         'event',
         'starting create',
-        'salt/cloud/{0}/creating'.format(vm_['name']),
+        'salt/cloud/{0}/creating'.format(name),
         {
-            'name': vm_['name'],
+            'name': name,
             'profile': vm_['profile'],
             'provider': vm_['driver'],
         },
         transport=__opts__['transport']
     )
 
-    log.info('Creating Cloud VM {0}'.format(vm_['name']))
+    log.info('Creating Cloud VM {0}'.format(name))
     conn = get_conn()
     kwargs = {
-        'hostname': vm_['name'],
-        'domain': vm_['domain'],
+        'hostname': hostname,
+        'domain': domain,
         'startCpus': vm_['cpu_number'],
         'maxMemory': vm_['ram'],
         'hourlyBillingFlag': vm_['hourly_billing'],
@@ -309,7 +324,7 @@ def create(vm_):
                             'The first 5 disks will be applied to the VM, '
                             'but the remaining disks will be ignored.\n'
                             'Please adjust your cloud configuration to only '
-                            'specify a maximum of 5 disks.'.format(vm_['name']))
+                            'specify a maximum of 5 disks.'.format(name))
                 break
 
     elif 'global_identifier' in vm_:
@@ -364,7 +379,7 @@ def create(vm_):
     salt.utils.cloud.fire_event(
         'event',
         'requesting instance',
-        'salt/cloud/{0}/requesting'.format(vm_['name']),
+        'salt/cloud/{0}/requesting'.format(name),
         {'kwargs': kwargs},
         transport=__opts__['transport']
     )
@@ -376,7 +391,7 @@ def create(vm_):
             'Error creating {0} on SoftLayer\n\n'
             'The following exception was thrown when trying to '
             'run the initial deployment: \n{1}'.format(
-                vm_['name'], str(exc)
+                name, str(exc)
             ),
             # Show the traceback if the debug logging level is enabled
             exc_info_on_loglevel=logging.DEBUG
@@ -390,7 +405,7 @@ def create(vm_):
     private_wds = config.get_cloud_config_value(
         'private_windows', vm_, __opts__, default=False
     )
-    if private_ssh or private_wds:
+    if private_ssh or private_wds or public_vlan is None or public_vlan is False:
         ip_type = 'primaryBackendIpAddress'
 
     def wait_for_ip():
@@ -398,8 +413,8 @@ def create(vm_):
         Wait for the IP address to become available
         '''
         nodes = list_nodes_full()
-        if ip_type in nodes[vm_['name']]:
-            return nodes[vm_['name']][ip_type]
+        if ip_type in nodes[hostname]:
+            return nodes[hostname][ip_type]
         time.sleep(1)
         return False
 
@@ -409,7 +424,7 @@ def create(vm_):
             'wait_for_fun_timeout', vm_, __opts__, default=15 * 60),
     )
     if config.get_cloud_config_value('deploy', vm_, __opts__) is not True:
-        return show_instance(vm_['name'], call='action')
+        return show_instance(hostname, call='action')
 
     SSH_PORT = 22
     WINDOWS_DS_PORT = 445
@@ -476,9 +491,9 @@ def create(vm_):
     salt.utils.cloud.fire_event(
         'event',
         'created instance',
-        'salt/cloud/{0}/created'.format(vm_['name']),
+        'salt/cloud/{0}/created'.format(name),
         {
-            'name': vm_['name'],
+            'name': name,
             'profile': vm_['profile'],
             'provider': vm_['driver'],
         },
@@ -501,7 +516,8 @@ def list_nodes_full(mask='mask[id]', call=None):
     conn = get_conn(service='SoftLayer_Account')
     response = conn.getVirtualGuests()
     for node_id in response:
-        ret[node_id['hostname']] = node_id
+        hostname = node_id['hostname'].split('.')[0]
+        ret[hostname] = node_id
     salt.utils.cloud.cache_node_list(ret, __active_provider_name__.split(':')[0], __opts__)
     return ret
 
@@ -585,6 +601,9 @@ def destroy(name, call=None):
         transport=__opts__['transport']
     )
 
+    # If the VM was created with use_fqdn, the short hostname will be used instead.
+    name = name.split('.')[0]
+
     node = show_instance(name, call='action')
     conn = get_conn()
     response = conn.deleteObject(id=node['id'])
