@@ -1,6 +1,12 @@
 # -*- coding: utf-8 -*-
 '''
 Service support for RHEL-based systems, including support for both upstart and sysvinit
+
+.. important::
+    If you feel that Salt should be using this module to manage services on a
+    minion, and it is using a different module (or gives an error similar to
+    *'service.start' is not available*), see :ref:`here
+    <module-provider-override>`.
 '''
 from __future__ import absolute_import
 
@@ -60,20 +66,29 @@ def __virtual__():
     if __grains__['os'] in enable:
         if __grains__['os'] == 'XenServer':
             return __virtualname__
-        try:
-            osrelease = float(__grains__.get('osrelease', 0))
-        except ValueError:
-            return (False, 'Cannot load rh_service module: '
-                           'osrelease grain, {0}, not a float,'.format(osrelease))
+
         if __grains__['os'] == 'SUSE':
-            if osrelease > 11:
-                return (False, 'Cannot load rh_service module on SUSE >= 11')
+            if str(__grains__['osrelease']).startswith('11'):
+                return __virtualname__
+            else:
+                return (False, 'Cannot load rh_service module on SUSE > 11')
+
+        osrelease_major = __grains__.get('osrelease_info', [0])[0]
+
         if __grains__['os'] == 'Fedora':
-            if osrelease > 15:
-                return (False, 'Cannot load rh_service module on Fedora >= 15')
-        if __grains__['os'] in ('RedHat', 'CentOS', 'ScientificLinux', 'OEL'):
-            if osrelease >= 7:
-                return (False, 'Cannot load rh_service module on RedHat >= 7')
+            if osrelease_major >= 15:
+                return (
+                    False,
+                    'Fedora >= 15 uses systemd, will not load rh_service.py '
+                    'as virtual \'service\''
+                )
+        if __grains__['os'] in ('RedHat', 'CentOS', 'ScientificLinux', 'OEL', 'CloudLinux'):
+            if osrelease_major >= 7:
+                return (
+                    False,
+                    'RedHat-based distros >= version 7 use systemd, will not '
+                    'load rh_service.py as virtual \'service\''
+                )
         return __virtualname__
     return (False, 'Cannot load rh_service module: OS not in {0}'.format(enable))
 
@@ -199,6 +214,28 @@ def _sysv_disable(name):
     return not __salt__['cmd.retcode'](cmd, python_shell=False)
 
 
+def _sysv_delete(name):
+    '''
+    Delete the named sysv service from the system. The service will be
+    deleted using chkconfig.
+    '''
+    if not _service_is_chkconfig(name):
+        return False
+    cmd = '/sbin/chkconfig --del {0}'.format(name)
+    return not __salt__['cmd.retcode'](cmd)
+
+
+def _upstart_delete(name):
+    '''
+    Delete an upstart service. This will only rename the .conf file
+    '''
+    if HAS_UPSTART:
+        if os.path.exists('/etc/init/{0}.conf'.format(name)):
+            os.rename('/etc/init/{0}.conf'.format(name),
+                      '/etc/init/{0}.conf.removed'.format(name))
+    return True
+
+
 def _upstart_services():
     '''
     Return list of upstart services.
@@ -214,9 +251,17 @@ def _sysv_services():
     '''
     Return list of sysv services.
     '''
-    ret = []
-    return [name for name in os.listdir('/etc/init.d')
-        if _service_is_sysv(name)]
+    _services = []
+    output = __salt__['cmd.run'](['chkconfig', '--list'], python_shell=False)
+    for line in output.splitlines():
+        comps = line.split()
+        try:
+            if comps[1].startswith('0:'):
+                _services.append(comps[0])
+        except IndexError:
+            continue
+    # Return only the services that have an initscript present
+    return [x for x in _services if _service_is_sysv(x)]
 
 
 def get_enabled(limit=''):
@@ -435,6 +480,24 @@ def status(name, sig=None):
     return __salt__['cmd.retcode'](cmd, python_shell=False, ignore_retcode=True) == 0
 
 
+def delete(name, **kwargs):
+    '''
+    Delete the named service
+
+    .. versionadded:: 2016.3
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' service.delete <service name>
+    '''
+    if _service_is_upstart(name):
+        return _upstart_delete(name)
+    else:
+        return _sysv_delete(name)
+
+
 def enable(name, **kwargs):
     '''
     Enable the named service to start at boot
