@@ -10,10 +10,12 @@ import logging
 
 # Import salt libs
 import salt.loader
-import salt.utils
-import salt.state
 import salt.runner
+import salt.state
+import salt.utils
+import salt.utils.schema as S
 from salt.utils.doc import strip_rst as _strip_rst
+from salt.ext.six.moves import zip
 
 # Import 3rd-party libs
 import salt.ext.six as six
@@ -79,9 +81,9 @@ def doc(*args):
         else:
             target_mod = ''
         if _use_fnmatch:
-            for fun in fnmatch.filter(__salt__.keys(), target_mod):  # pylint: disable=incompatible-py3-code
-                docs[fun] = __salt__[fun].__doc__                    # There's no problem feeding fnmatch.filter()
-        else:                                                        # with a Py3's dict_keys() instance
+            for fun in fnmatch.filter(__salt__.keys(), target_mod):
+                docs[fun] = __salt__[fun].__doc__
+        else:
 
             for fun in __salt__:
                 if fun == module or fun.startswith(target_mod):
@@ -313,9 +315,8 @@ def renderer_doc(*args):
 
     for module in args:
         if '*' in module:
-            for fun in fnmatch.filter(renderers_.keys(), module):   # pylint: disable=incompatible-py3-code
-                docs[fun] = renderers_[fun].__doc__                 # There's no problem feeding fnmatch.filter()
-                                                                    # with a Py3's dict_keys() instance
+            for fun in fnmatch.filter(renderers_.keys(), module):
+                docs[fun] = renderers_[fun].__doc__
         else:
             for fun in six.iterkeys(renderers_):
                 docs[fun] = renderers_[fun].__doc__
@@ -745,9 +746,9 @@ def list_returners(*args):
         return sorted(returners)
 
     for module in args:
-        for func in fnmatch.filter(returners_.keys(), module):  # pylint: disable=incompatible-py3-code
-            comps = func.split('.')                             # There's no problem feeding fnmatch.filter()
-            if len(comps) < 2:                                  # with a Py3's dict_keys() instance
+        for func in fnmatch.filter(returners_.keys(), module):
+            comps = func.split('.')
+            if len(comps) < 2:
                 continue
             returners.add(comps[0])
     return sorted(returners)
@@ -837,3 +838,62 @@ def list_renderers(*args):
         for func in fnmatch.filter(ren_, module):
             ren.add(func)
     return sorted(ren)
+
+
+def _argspec_to_schema(mod, spec):
+    args = spec['args']
+    defaults = spec['defaults'] or []
+
+    args_req = args[:len(args) - len(defaults)]
+    args_defaults = list(zip(args[-len(defaults):], defaults))
+
+    types = {
+        'title': mod,
+        'description': mod,
+    }
+
+    for i in args_req:
+        types[i] = S.OneOfItem(items=(
+            S.BooleanItem(title=i, description=i, required=True),
+            S.IntegerItem(title=i, description=i, required=True),
+            S.NumberItem(title=i, description=i, required=True),
+            S.StringItem(title=i, description=i, required=True),
+
+            # S.ArrayItem(title=i, description=i, required=True),
+            # S.DictItem(title=i, description=i, required=True),
+        ))
+
+    for i, j in args_defaults:
+        types[i] = S.OneOfItem(items=(
+            S.BooleanItem(title=i, description=i, default=j),
+            S.IntegerItem(title=i, description=i, default=j),
+            S.NumberItem(title=i, description=i, default=j),
+            S.StringItem(title=i, description=i, default=j),
+
+            # S.ArrayItem(title=i, description=i, default=j),
+            # S.DictItem(title=i, description=i, default=j),
+        ))
+
+    return type(mod, (S.Schema,), types).serialize()
+
+
+def state_schema(module=''):
+    '''
+    Return a JSON Schema for the given state function(s)
+
+    .. versionadded:: 2016.3.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' sys.state_schema
+        salt '*' sys.state_schema pkg.installed
+    '''
+    specs = state_argspec(module)
+
+    schemas = []
+    for state_mod, state_spec in specs.items():
+        schemas.append(_argspec_to_schema(state_mod, state_spec))
+
+    return schemas
