@@ -52,7 +52,11 @@ def __virtual__():
     if __grains__['kernel'] == 'Linux' \
             and salt.utils.systemd.booted(__context__):
         return __virtualname__
-    return False
+    return (
+        False,
+        'The systemd execution module failed to load: only available on Linux '
+        'systems which have been booted with systemd.'
+    )
 
 
 def _canonical_unit_name(name):
@@ -60,6 +64,8 @@ def _canonical_unit_name(name):
     Build a canonical unit name treating unit names without one
     of the valid suffixes as a service.
     '''
+    if not isinstance(name, six.string_types):
+        name = str(name)
     if any(name.endswith(suffix) for suffix in VALID_UNIT_TYPES):
         return name
     return '%s.service' % name
@@ -273,7 +279,11 @@ def _sysv_enabled(name):
     (starts with "S") to its script is found in /etc/init.d in the current
     runlevel.
     '''
-    return bool(glob.glob('/etc/rc%s.d/S*%s' % (_runlevel(), name)))
+    # Find exact match (disambiguate matches like "S01anacron" for cron)
+    for match in glob.glob('/etc/rc%s.d/S*%s' % (_runlevel(), name)):
+        if re.match(r'S\d{,2}%s' % name, os.path.basename(match)):
+            return True
+    return False
 
 
 def _untracked_custom_unit_found(name):
@@ -550,7 +560,10 @@ def mask(name, runtime=False):
                                   redirect_stderr=True)
 
     if out['retcode'] != 0:
-        raise CommandExecutionError('Failed to mask service \'%s\'' % name)
+        raise CommandExecutionError(
+            'Failed to mask service \'%s\'' % name,
+            info=out['stdout']
+        )
 
     return True
 
@@ -592,10 +605,10 @@ def start(name):
 
         salt '*' service.start <service name>
     '''
-    if _untracked_custom_unit_found(name) or _unit_file_changed(name):
-        systemctl_reload()
+    _check_for_unit_changes(name)
     unmask(name)
-    return not __salt__['cmd.retcode'](_systemctl_cmd('start', name))
+    return __salt__['cmd.retcode'](_systemctl_cmd('start', name),
+                                   python_shell=False) == 0
 
 
 def stop(name):
@@ -667,8 +680,8 @@ def force_reload(name):
 # established by Salt's service management states.
 def status(name, sig=None):  # pylint: disable=unused-argument
     '''
-    Return the status for a service via systemd, returns a bool
-    whether the service is running.
+    Return the status for a service via systemd, returns ``True`` if the
+    service is running and ``False`` if it is not.
 
     CLI Example:
 
@@ -772,7 +785,7 @@ def enabled(name, **kwargs):  # pylint: disable=unused-argument
 
 def disabled(name):
     '''
-    Return if the named service is disabled to start on boot
+    Return if the named service is disabled from starting on boot
 
     CLI Example:
 
@@ -794,7 +807,9 @@ def show(name):
         salt '*' service.show <service name>
     '''
     ret = {}
-    for line in __salt__['cmd.run'](_systemctl_cmd('show', name)).splitlines():
+    out = __salt__['cmd.run'](_systemctl_cmd('show', name),
+                              python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         comps = line.split('=')
         name = comps[0]
         value = '='.join(comps[1:])
@@ -822,11 +837,10 @@ def execs():
 
         salt '*' service.execs
     '''
-    execs_ = {}
+    ret = {}
     for service in get_all():
         data = show(service)
         if 'ExecStart' not in data:
             continue
-        execs_[service] = data['ExecStart']['path']
-
-    return execs_
+        ret[service] = data['ExecStart']['path']
+    return ret
