@@ -16,7 +16,7 @@ For remote signing, peers must be permitted to remotely call the
 :mod:`sign_remote_certificate <salt.modules.x509.sign_remote_certificate>` function.
 
 
-/etc/salt/master.d/peer.sls
+/etc/salt/master.d/peer.conf
 
 .. code-block:: yaml
 
@@ -125,6 +125,7 @@ handle properly formatting the text before writing the output.
 /srv/salt/cert.sls
 
 .. code-block:: yaml
+
     /usr/local/share/ca-certificates:
       file.directory: []
 
@@ -169,6 +170,16 @@ import salt.utils
 import salt.ext.six as six
 
 
+def __virtual__():
+    '''
+    only load this module if the corresponding execution module is loaded
+    '''
+    if 'x509.get_pem_entry' in __salt__:
+        return 'x509'
+    else:
+        return (False, 'Could not load x509 state: m2crypto unavailable')
+
+
 def _revoked_to_list(revs):
     '''
     Turn the mess of OrderedDicts and Lists into a list of dicts for
@@ -194,7 +205,7 @@ def private_key_managed(name,
                         new=False,
                         backup=False):
     '''
-    Manage a private key's existance.
+    Manage a private key's existence.
 
     name:
         Path to the private key
@@ -336,11 +347,11 @@ def certificate_managed(name,
         Path to the certificate
 
     days_remaining:
-        The minimum number of days remaining when the certificate should be recreted. Default is 90. A
+        The minimum number of days remaining when the certificate should be recreated. Default is 90. A
         value of 0 disables automatic renewal.
 
     backup:
-        When replacing an existing file, backup the old file onthe minion. Default is False.
+        When replacing an existing file, backup the old file on the minion. Default is False.
 
     kwargs:
         Any arguments supported by :mod:`x509.create_certificate <salt.modules.x509.create_certificate>`
@@ -381,6 +392,9 @@ def certificate_managed(name,
     '''
     ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
 
+    if 'path' in kwargs:
+        name = kwargs.pop('path')
+
     current_days_remaining = 0
     current_comp = {}
 
@@ -613,6 +627,9 @@ def pem_managed(name,
         ret['comment'] = 'The file is already in the correct state'
         return ret
 
+    ret['changes']['new'] = new
+    ret['changes']['old'] = current
+
     if __opts__['test'] is True:
         ret['result'] = None
         ret['comment'] = 'The file {0} will be updated.'.format(name)
